class Individual {

  /*
    Note on the type modifiers used here:
    final: Value cannot be changed after initialization
    static: Value is not attached to a specific instance
            To reference a static variable outside of
            this class use: Individual.<variable name>
            i.e. Indivudal.CHROMOSOME_LENGTH
  */
  static final int CHROMOSOME_LENGTH = 6;
  static final int SIDES_GENE_LENGTH = 5;
  static final int SIZE_GENE_LENGTH = 6;
  static final int SPIN_GENE_LENGTH = 4;
  static final int COLOR_GENE_LENGTH = 8;

  static final int SIDES_IND = 0;
  static final int SIZE_IND = 1;
  static final int SPIN_IND = 2;
  static final int RED_IND = 3;
  static final int GREEN_IND = 4;
  static final int BLUE_IND = 5;

  RegularGon phenotype;
  Gene[] chromosome;
  float fitness;


  Individual(boolean random) {

    chromosome = new Gene[CHROMOSOME_LENGTH];

    if (random) {
      chromosome[SIDES_IND] = new Gene(SIDES_GENE_LENGTH);
      chromosome[SIZE_IND] = new Gene(SIZE_GENE_LENGTH);
      chromosome[SPIN_IND] = new Gene(SPIN_GENE_LENGTH);
      chromosome[RED_IND] = new Gene(COLOR_GENE_LENGTH);
      chromosome[GREEN_IND] = new Gene(COLOR_GENE_LENGTH);
      chromosome[BLUE_IND] = new Gene(COLOR_GENE_LENGTH);

      setPhenotype();
    }
  }//constructor

  void setPhenotype() {
    int sides = chromosome[SIDES_IND].getValue();
    int siz = chromosome[SIZE_IND].getValue();
    float spin = chromosome[SPIN_IND].getValue() - 7;
    color c = color(chromosome[RED_IND].getValue(), chromosome[GREEN_IND].getValue(), chromosome[BLUE_IND].getValue());
    phenotype = new RegularGon(sides, siz, spin, c);
  }

  /*
    creates/returns a new individual object from mating this and partnet

    For each gene, randomly select which parent to take it from */
  Individual mate(Individual partner) {
    Individual child = new Individual(false); //false: don't initialize genes
    Individual first, second;
    //Randomize who starts
    if (random(2) < 1) { first = this; second = partner; }
    else               { first = partner; second = this; }
    
    //All genes >= crossover come from second
    //we want at least 1 from each parent
    //crossover should be in 1..length-1, inclusive
    int crossoverAt = int(random(1,chromosome.length));//wont include length
    
    //Copy genes over
    for (int i = 0; i < chromosome.length; i++) {
      Gene pickedGene;
      if (i< crossoverAt)    { pickedGene = first.chromosome[i]; }
      else                   { pickedGene = second.chromosome[i]; }
      
      child.chromosome[i] = new Gene(pickedGene); //use copy constructor
    }
    child.setPhenotype();
    return child;
  }//mate

/*  Set the fitness based on how close the calling
    object is to the target. */
  void setFitness(Individual target) {
    //let's make fitness be how many bits match
    int totalBits = 0;
    int matchingBits = 0;
    
    for(int gene_i = 0; gene_i < chromosome.length; gene_i++) {
      Gene g1 = this.chromosome[gene_i];
      Gene g2 = target.chromosome[gene_i];
      for (int bit_i = 0; bit_i < g1.genotype.length; bit_i++) {
        totalBits++;
        if(g1.genotype[bit_i] == g2.genotype[bit_i])
          { matchingBits++; }
      }
    }
    
    fitness = float(matchingBits) / totalBits;
    fitness = pow(fitness, FITNESS_EXPONENT); //raise fitness to a power to steepen the fitness function
  }//setFitness

  void mutate(float rate) {
    for (int g=0; g < chromosome.length; g++) {
      if (random(1) < rate) {
        chromosome[g].mutate();
      }
    }
    setPhenotype();
  }//mutate

  void display(int x, int y, boolean showFitness) {
    phenotype.display(x, y);
    if (showFitness) {

      //println(fitness);
      textSize(15);
      fill(0);
      textAlign(CENTER);
      text(fitness, x, y);
    }
  }//display()

  String toString() {
    String s = "<Individual:\n";
    for (int i=0; i<CHROMOSOME_LENGTH; i++) {
      s+= "  " + chromosome[i] + "\n";
    }
    s+= "  fitness: " + fitness + ">";
    return s;
  }//toString()

}//Individual
