class Population {
  
  Individual[] pop;
  //target will be pop[0]
  
  float totalFitness;
  float mutationRate;


  Population(int popSize) {
    pop = new Individual[popSize];
    //don't construct individuals yet
    mutationRate = 0.05;
  }
  
  void randomPop() {
    for(int i = 0; i < pop.length; i++) {
      pop[i] = new Individual(true);
    }
  }
  
  Individual get(int i) { return pop[i]; }
  
  void setFitness(Individual target) {
    totalFitness = 0;
    for(int i = 0; i < pop.length; i++) { 
      pop[i].setFitness(target);
      if(i != 0) //Omit 0th individual, it's not properly part of the pop
        { totalFitness += pop[i].fitness; }
    }
  }
  
  int getBestIndex() {
    float bestFit = pop[1].fitness;
    int bestI = 1;
    
    //Start at 1, omit target pop[0]
    for(int i = 1; i < pop.length; i++) {
      if(pop[i].fitness > bestFit) { 
        bestFit = pop[i].fitness;
        bestI = i;
      }
    }
    return bestI;
  }
  float getBestFitness() {
    return pop[getBestIndex()].fitness;
  }
  float getAverageFitness() {
    return totalFitness / (pop.length - 1); // totalFitness excludes pop[0]
  }
  
  Individual select() {
    // Return a random Individual from pop array
    // chosen in proportion to Individual fitness relative to totalFitness
    
    // Roullette wheel select:
    // pick select value from [0 to totalFitness)
    // loop over individuals, subtracting from select value by their fitness
    // if individual fitness < selectVal, they are chosen
    
    float selVal = random(0, totalFitness);
    //println("Starting roulette: totalFitness = ", totalFitness, " selval=", selVal);
    
    //Start from 1: omit 0th indiv from selection, since it's the target
    for(int i = 1; i < pop.length; i++) {
      //println("roulette ", i, ", remaining selval=",selVal,"  ith fitness=", pop[i].fitness);
      selVal -= pop[i].fitness;
      
      if (selVal < 0) {
        //select fell within this individual's slot on wheel
        return pop[i];
      }
    }
    
    //We should never get to this point unless
    //There's a bug in my roulette selection code,
    //In which case crash so I can see the error and to shut Java up
    throw new RuntimeException();
    //Could also do something like println("!!!ERROR"); return null;
  }
  
  
  Population matingSeason() {
    //Create a new population of same size
    //Copy over individual 0

    Population nextGen = new Population(pop.length);
    nextGen.pop[0] = this.pop[0];   //Copy target
    nextGen.pop[1] = this.pop[getBestIndex()]; //Copy best
    nextGen.mutationRate = this.mutationRate;
    
    //Select 2 members of current pop, mate them to create a child
    //mutate the child with this.mutationRate
    //add the child to the new pop
    for (int i = 2; i < pop.length; i++) { //skip 0th for target, 1st is best
      Individual i1, i2, child;
      i1 = this.select();
      i2 = this.select();
      child = i1.mate(i2);
      child.mutate(mutationRate);
      nextGen.pop[i] = child;
    }
    
    //start them all spinning the same way for neatness
    for(int i = 0; i < pop.length; i++) { 
      nextGen.pop[i].phenotype.displayAngle = 0;
    }
    
    return nextGen;
  }
  
  
  void drawPopGrid(int cols, int rows, int gridSize, int offset, boolean showFitness) {
    //Don't actually need rows if I'm doing it this way
    for (int i = 0; i < pop.length; i++) {
      int x = (i % cols) * (gridSize + offset) + (gridSize / 2);
      int y = (i / cols) * (gridSize + offset) + (gridSize / 2);
      pop[i].display(x, y, showFitness);
    }
  }
  
    void drawHighlight(int cols, int rows, int gridSize, int offset, int index, color c) {
      int x = (index % cols) * (gridSize + offset);
      int y = (index / cols) * (gridSize + offset);
      strokeWeight(5);
      noFill();
      stroke(c);
      rect(x,y, gridSize, gridSize);
  }
}
